"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var common_1 = require("./common");
function validate(updateObj, graphQLType) {
    validateNonNullableFields(updateObj, graphQLType);
}
function validateNonNullableFields(updateObj, graphQLType) {
    if (!updateObj.$setOnInsert)
        return;
    var requiredFields = getNonNullableFields(graphQLType);
    var updatedFields = Object.keys(Object.assign.apply(Object, [{}].concat(Object.values(updateObj))));
    var missingFields = requiredFields.filter(function (requiredField) { return !updatedFields.includes(requiredField); });
    if (missingFields.length)
        throw "Missing Non-nullable field: " + missingFields.join(', ');
}
function getNonNullableFields(graphQLType) {
    var fields = graphQLType.getFields();
    return Object.keys(fields)
        .filter(function (key) { return common_1.isNonNullType(fields[key].type); })
        .reduce(function (agg, key) {
        var field = fields[key];
        var innerType = common_1.getInnerType(field.type);
        var isScalar = common_1.isScalarType(innerType);
        if (isScalar)
            return agg.concat([key]);
        var nestedNonNullableFields = getNonNullableFields(innerType);
        return agg.concat(nestedNonNullableFields.map(function (_) { return key + "." + _; }));
    }, []);
}
exports.default = validate;
